import os

import numpy as np


class Model(object):
    def __init__(self, r, n, m, k):
        self.r = r
        self.r2 = r ** 2
        self.m = m
        self.k = k
        self.large_balls = np.random.uniform(0.0, 1.0, size=(n, 3))
        self.small_balls = np.zeros((m, 3))
        initial_ball = np.random.uniform(0.0, 1.0, size=(3, ))
        self.small_balls[:, :] = initial_ball

    def add_balls(self):
        for i in range(self.m):
            self.add_ball(i)

    def add_ball(self, i):
        cands = []
        distances = []
        for j in range(self.k):
            cand = self.new_cand()
            cands.append(cand)
            distances.append(np.min(np.sum((self.small_balls - cand) ** 2, axis=1)))
        self.small_balls[i, :] = cands[np.argmax(distances)]

    def new_cand(self):
        while True:
            cand = np.random.uniform(0.0, 1.0, size=(3, ))
            if np.all(np.sum((self.large_balls - cand) ** 2, axis=1) > self.r2):
                return cand


def test_main():
    model = Model(0.4, 2, 1000, 20)
    model.add_balls()
    np.savetxt("sballs.txt", model.small_balls)
    np.savetxt("lballs.txt", model.large_balls)


def main():
    # param_1 = [0.12, 20, 1000, 10]
    # param_2 = [0.14, 20, 1000, 10]
    os.makedirs("pc", exist_ok=True)
    os.makedirs("pc-test", exist_ok=True)
    param_1 = [0.12, 20, 1000, 10]
    param_2 = [0.13, 20, 1000, 10]

    for i in range(0, 100):
        print(i)
        model = Model(*param_1)
        model.add_balls()
        np.savetxt("pc/{:04d}.txt".format(i), model.small_balls)

    # for i in range(0, 50):
    #     print(i)
    #     model = Model(*param_1)
    #     model.add_balls()
    #     np.savetxt("pc-test/{:04d}.txt".format(i), model.small_balls)

    for i in range(100, 200):
        print(i)
        model = Model(*param_2)
        model.add_balls()
        np.savetxt("pc/{:04d}.txt".format(i), model.small_balls)

    # for i in range(50, 100):
    #     print(i)
    #     model = Model(*param_2)
    #     model.add_balls()
    #     np.savetxt("pc-test/{:04d}.txt".format(i), model.small_balls)

    np.savetxt("pc/label.txt", [0] * 100 + [1] * 100, fmt="%d")
    np.savetxt("pc-test/label.txt", [0] * 50 + [1] * 50, fmt="%d")


main()
